'use strict';

const fs = require('fs');
const events = require('events');
const chalk = require('chalk');
const debug = require('debug');
const path = require('path');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
const chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
const debug__default = /*#__PURE__*/_interopDefaultLegacy(debug);
const path__default = /*#__PURE__*/_interopDefaultLegacy(path);

function toArr(any) {
	return any == null ? [] : Array.isArray(any) ? any : [any];
}

function toVal(out, key, val, opts) {
	var x, old=out[key], nxt=(
		!!~opts.string.indexOf(key) ? (val == null || val === true ? '' : String(val))
		: typeof val === 'boolean' ? val
		: !!~opts.boolean.indexOf(key) ? (val === 'false' ? false : val === 'true' || (out._.push((x = +val,x * 0 === 0) ? x : val),!!val))
		: (x = +val,x * 0 === 0) ? x : val
	);
	out[key] = old == null ? nxt : (Array.isArray(old) ? old.concat(nxt) : [old, nxt]);
}

function mri2 (args, opts) {
	args = args || [];
	opts = opts || {};

	var k, arr, arg, name, val, out={ _:[] };
	var i=0, j=0, idx=0, len=args.length;

	const alibi = opts.alias !== void 0;
	const strict = opts.unknown !== void 0;
	const defaults = opts.default !== void 0;

	opts.alias = opts.alias || {};
	opts.string = toArr(opts.string);
	opts.boolean = toArr(opts.boolean);

	if (alibi) {
		for (k in opts.alias) {
			arr = opts.alias[k] = toArr(opts.alias[k]);
			for (i=0; i < arr.length; i++) {
				(opts.alias[arr[i]] = arr.concat(k)).splice(i, 1);
			}
		}
	}

	for (i=opts.boolean.length; i-- > 0;) {
		arr = opts.alias[opts.boolean[i]] || [];
		for (j=arr.length; j-- > 0;) opts.boolean.push(arr[j]);
	}

	for (i=opts.string.length; i-- > 0;) {
		arr = opts.alias[opts.string[i]] || [];
		for (j=arr.length; j-- > 0;) opts.string.push(arr[j]);
	}

	if (defaults) {
		for (k in opts.default) {
			name = typeof opts.default[k];
			arr = opts.alias[k] = opts.alias[k] || [];
			if (opts[name] !== void 0) {
				opts[name].push(k);
				for (i=0; i < arr.length; i++) {
					opts[name].push(arr[i]);
				}
			}
		}
	}

	const keys = strict ? Object.keys(opts.alias) : [];

	for (i=0; i < len; i++) {
		arg = args[i];

		if (arg === '--') {
			out._ = out._.concat(args.slice(++i));
			break;
		}

		for (j=0; j < arg.length; j++) {
			if (arg.charCodeAt(j) !== 45) break; // "-"
		}

		if (j === 0) {
			out._.push(arg);
		} else if (arg.substring(j, j + 3) === 'no-') {
			name = arg.substring(j + 3);
			if (strict && !~keys.indexOf(name)) {
				return opts.unknown(arg);
			}
			out[name] = false;
		} else {
			for (idx=j+1; idx < arg.length; idx++) {
				if (arg.charCodeAt(idx) === 61) break; // "="
			}

			name = arg.substring(j, idx);
			val = arg.substring(++idx) || (i+1 === len || (''+args[i+1]).charCodeAt(0) === 45 || args[++i]);
			arr = (j === 2 ? [name] : name);

			for (idx=0; idx < arr.length; idx++) {
				name = arr[idx];
				if (strict && !~keys.indexOf(name)) return opts.unknown('-'.repeat(j) + name);
				toVal(out, name, (idx + 1 < arr.length) || val, opts);
			}
		}
	}

	if (defaults) {
		for (k in opts.default) {
			if (out[k] === void 0) {
				out[k] = opts.default[k];
			}
		}
	}

	if (alibi) {
		for (k in out) {
			arr = opts.alias[k] || [];
			while (arr.length > 0) {
				out[arr.shift()] = out[k];
			}
		}
	}

	return out;
}

const removeBrackets = (v) => v.replace(/[<[].+/, "").trim();
const findAllBrackets = (v) => {
  const ANGLED_BRACKET_RE_GLOBAL = /<([^>]+)>/g;
  const SQUARE_BRACKET_RE_GLOBAL = /\[([^\]]+)\]/g;
  const res = [];
  const parse = (match) => {
    let variadic = false;
    let value = match[1];
    if (value.startsWith("...")) {
      value = value.slice(3);
      variadic = true;
    }
    return {
      required: match[0].startsWith("<"),
      value,
      variadic
    };
  };
  let angledMatch;
  while (angledMatch = ANGLED_BRACKET_RE_GLOBAL.exec(v)) {
    res.push(parse(angledMatch));
  }
  let squareMatch;
  while (squareMatch = SQUARE_BRACKET_RE_GLOBAL.exec(v)) {
    res.push(parse(squareMatch));
  }
  return res;
};
const getMriOptions = (options) => {
  const result = {alias: {}, boolean: []};
  for (const [index, option] of options.entries()) {
    if (option.names.length > 1) {
      result.alias[option.names[0]] = option.names.slice(1);
    }
    if (option.isBoolean) {
      if (option.negated) {
        const hasStringTypeOption = options.some((o, i) => {
          return i !== index && o.names.some((name) => option.names.includes(name)) && typeof o.required === "boolean";
        });
        if (!hasStringTypeOption) {
          result.boolean.push(option.names[0]);
        }
      } else {
        result.boolean.push(option.names[0]);
      }
    }
  }
  return result;
};
const findLongest = (arr) => {
  return arr.sort((a, b) => {
    return a.length > b.length ? -1 : 1;
  })[0];
};
const padRight = (str, length) => {
  return str.length >= length ? str : `${str}${" ".repeat(length - str.length)}`;
};
const camelcase = (input) => {
  return input.replace(/([a-z])-([a-z])/g, (_, p1, p2) => {
    return p1 + p2.toUpperCase();
  });
};
const setDotProp = (obj, keys, val) => {
  let i = 0;
  let length = keys.length;
  let t = obj;
  let x;
  for (; i < length; ++i) {
    x = t[keys[i]];
    t = t[keys[i]] = i === length - 1 ? val : x != null ? x : !!~keys[i + 1].indexOf(".") || !(+keys[i + 1] > -1) ? {} : [];
  }
};
const setByType = (obj, transforms) => {
  for (const key of Object.keys(transforms)) {
    const transform = transforms[key];
    if (transform.shouldTransform) {
      obj[key] = Array.prototype.concat.call([], obj[key]);
      if (typeof transform.transformFunction === "function") {
        obj[key] = obj[key].map(transform.transformFunction);
      }
    }
  }
};
const getFileName = (input) => {
  const m = /([^\\\/]+)$/.exec(input);
  return m ? m[1] : "";
};
const camelcaseOptionName = (name) => {
  return name.split(".").map((v, i) => {
    return i === 0 ? camelcase(v) : v;
  }).join(".");
};
class CACError extends Error {
  constructor(message) {
    super(message);
    this.name = this.constructor.name;
    if (typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(this, this.constructor);
    } else {
      this.stack = new Error(message).stack;
    }
  }
}

class Option {
  constructor(rawName, description, config) {
    this.rawName = rawName;
    this.description = description;
    this.config = Object.assign({}, config);
    rawName = rawName.replace(/\.\*/g, "");
    this.negated = false;
    this.names = removeBrackets(rawName).split(",").map((v) => {
      let name = v.trim().replace(/^-{1,2}/, "");
      if (name.startsWith("no-")) {
        this.negated = true;
        name = name.replace(/^no-/, "");
      }
      return camelcaseOptionName(name);
    }).sort((a, b) => a.length > b.length ? 1 : -1);
    this.name = this.names[this.names.length - 1];
    if (this.negated && this.config.default == null) {
      this.config.default = true;
    }
    if (rawName.includes("<")) {
      this.required = true;
    } else if (rawName.includes("[")) {
      this.required = false;
    } else {
      this.isBoolean = true;
    }
  }
}

const processArgs = process.argv;
const platformInfo = `${process.platform}-${process.arch} node-${process.version}`;

class Command {
  constructor(rawName, description, config = {}, cli) {
    this.rawName = rawName;
    this.description = description;
    this.config = config;
    this.cli = cli;
    this.options = [];
    this.aliasNames = [];
    this.name = removeBrackets(rawName);
    this.args = findAllBrackets(rawName);
    this.examples = [];
  }
  usage(text) {
    this.usageText = text;
    return this;
  }
  allowUnknownOptions() {
    this.config.allowUnknownOptions = true;
    return this;
  }
  ignoreOptionDefaultValue() {
    this.config.ignoreOptionDefaultValue = true;
    return this;
  }
  version(version, customFlags = "-v, --version") {
    this.versionNumber = version;
    this.option(customFlags, "Display version number");
    return this;
  }
  example(example) {
    this.examples.push(example);
    return this;
  }
  option(rawName, description, config) {
    const option = new Option(rawName, description, config);
    this.options.push(option);
    return this;
  }
  alias(name) {
    this.aliasNames.push(name);
    return this;
  }
  action(callback) {
    this.commandAction = callback;
    return this;
  }
  isMatched(name) {
    return this.name === name || this.aliasNames.includes(name);
  }
  get isDefaultCommand() {
    return this.name === "" || this.aliasNames.includes("!");
  }
  get isGlobalCommand() {
    return this instanceof GlobalCommand;
  }
  hasOption(name) {
    name = name.split(".")[0];
    return this.options.find((option) => {
      return option.names.includes(name);
    });
  }
  outputHelp() {
    const {name, commands} = this.cli;
    const {
      versionNumber,
      options: globalOptions,
      helpCallback
    } = this.cli.globalCommand;
    let sections = [
      {
        body: `${name}${versionNumber ? `/${versionNumber}` : ""}`
      }
    ];
    sections.push({
      title: "Usage",
      body: `  $ ${name} ${this.usageText || this.rawName}`
    });
    const showCommands = (this.isGlobalCommand || this.isDefaultCommand) && commands.length > 0;
    if (showCommands) {
      const longestCommandName = findLongest(commands.map((command) => command.rawName));
      sections.push({
        title: "Commands",
        body: commands.map((command) => {
          return `  ${padRight(command.rawName, longestCommandName.length)}  ${command.description}`;
        }).join("\n")
      });
      sections.push({
        title: `For more info, run any command with the \`--help\` flag`,
        body: commands.map((command) => `  $ ${name}${command.name === "" ? "" : ` ${command.name}`} --help`).join("\n")
      });
    }
    let options = this.isGlobalCommand ? globalOptions : [...this.options, ...globalOptions || []];
    if (!this.isGlobalCommand && !this.isDefaultCommand) {
      options = options.filter((option) => option.name !== "version");
    }
    if (options.length > 0) {
      const longestOptionName = findLongest(options.map((option) => option.rawName));
      sections.push({
        title: "Options",
        body: options.map((option) => {
          return `  ${padRight(option.rawName, longestOptionName.length)}  ${option.description} ${option.config.default === void 0 ? "" : `(default: ${option.config.default})`}`;
        }).join("\n")
      });
    }
    if (this.examples.length > 0) {
      sections.push({
        title: "Examples",
        body: this.examples.map((example) => {
          if (typeof example === "function") {
            return example(name);
          }
          return example;
        }).join("\n")
      });
    }
    if (helpCallback) {
      sections = helpCallback(sections) || sections;
    }
    console.log(sections.map((section) => {
      return section.title ? `${section.title}:
${section.body}` : section.body;
    }).join("\n\n"));
  }
  outputVersion() {
    const {name} = this.cli;
    const {versionNumber} = this.cli.globalCommand;
    if (versionNumber) {
      console.log(`${name}/${versionNumber} ${platformInfo}`);
    }
  }
  checkRequiredArgs() {
    const minimalArgsCount = this.args.filter((arg) => arg.required).length;
    if (this.cli.args.length < minimalArgsCount) {
      throw new CACError(`missing required args for command \`${this.rawName}\``);
    }
  }
  checkUnknownOptions() {
    const {options, globalCommand} = this.cli;
    if (!this.config.allowUnknownOptions) {
      for (const name of Object.keys(options)) {
        if (name !== "--" && !this.hasOption(name) && !globalCommand.hasOption(name)) {
          throw new CACError(`Unknown option \`${name.length > 1 ? `--${name}` : `-${name}`}\``);
        }
      }
    }
  }
  checkOptionValue() {
    const {options: parsedOptions, globalCommand} = this.cli;
    const options = [...globalCommand.options, ...this.options];
    for (const option of options) {
      const value = parsedOptions[option.name.split(".")[0]];
      if (option.required) {
        const hasNegated = options.some((o) => o.negated && o.names.includes(option.name));
        if (value === true || value === false && !hasNegated) {
          throw new CACError(`option \`${option.rawName}\` value is missing`);
        }
      }
    }
  }
}
class GlobalCommand extends Command {
  constructor(cli) {
    super("@@global@@", "", {}, cli);
  }
}

var __assign = Object.assign;
class CAC extends events.EventEmitter {
  constructor(name = "") {
    super();
    this.name = name;
    this.commands = [];
    this.rawArgs = [];
    this.args = [];
    this.options = {};
    this.globalCommand = new GlobalCommand(this);
    this.globalCommand.usage("<command> [options]");
  }
  usage(text) {
    this.globalCommand.usage(text);
    return this;
  }
  command(rawName, description, config) {
    const command = new Command(rawName, description || "", config, this);
    command.globalCommand = this.globalCommand;
    this.commands.push(command);
    return command;
  }
  option(rawName, description, config) {
    this.globalCommand.option(rawName, description, config);
    return this;
  }
  help(callback) {
    this.globalCommand.option("-h, --help", "Display this message");
    this.globalCommand.helpCallback = callback;
    this.showHelpOnExit = true;
    return this;
  }
  version(version, customFlags = "-v, --version") {
    this.globalCommand.version(version, customFlags);
    this.showVersionOnExit = true;
    return this;
  }
  example(example) {
    this.globalCommand.example(example);
    return this;
  }
  outputHelp() {
    if (this.matchedCommand) {
      this.matchedCommand.outputHelp();
    } else {
      this.globalCommand.outputHelp();
    }
  }
  outputVersion() {
    this.globalCommand.outputVersion();
  }
  setParsedInfo({args, options}, matchedCommand, matchedCommandName) {
    this.args = args;
    this.options = options;
    if (matchedCommand) {
      this.matchedCommand = matchedCommand;
    }
    if (matchedCommandName) {
      this.matchedCommandName = matchedCommandName;
    }
    return this;
  }
  unsetMatchedCommand() {
    this.matchedCommand = void 0;
    this.matchedCommandName = void 0;
  }
  parse(argv = processArgs, {
    run = true
  } = {}) {
    this.rawArgs = argv;
    if (!this.name) {
      this.name = argv[1] ? getFileName(argv[1]) : "cli";
    }
    let shouldParse = true;
    for (const command of this.commands) {
      const parsed = this.mri(argv.slice(2), command);
      const commandName = parsed.args[0];
      if (command.isMatched(commandName)) {
        shouldParse = false;
        const parsedInfo = __assign(__assign({}, parsed), {
          args: parsed.args.slice(1)
        });
        this.setParsedInfo(parsedInfo, command, commandName);
        this.emit(`command:${commandName}`, command);
      }
    }
    if (shouldParse) {
      for (const command of this.commands) {
        if (command.name === "") {
          shouldParse = false;
          const parsed = this.mri(argv.slice(2), command);
          this.setParsedInfo(parsed, command);
          this.emit(`command:!`, command);
        }
      }
    }
    if (shouldParse) {
      const parsed = this.mri(argv.slice(2));
      this.setParsedInfo(parsed);
    }
    if (this.options.help && this.showHelpOnExit) {
      this.outputHelp();
      run = false;
      this.unsetMatchedCommand();
    }
    if (this.options.version && this.showVersionOnExit && this.matchedCommandName == null) {
      this.outputVersion();
      run = false;
      this.unsetMatchedCommand();
    }
    const parsedArgv = {args: this.args, options: this.options};
    if (run) {
      this.runMatchedCommand();
    }
    if (!this.matchedCommand && this.args[0]) {
      this.emit("command:*");
    }
    return parsedArgv;
  }
  mri(argv, command) {
    const cliOptions = [
      ...this.globalCommand.options,
      ...command ? command.options : []
    ];
    const mriOptions = getMriOptions(cliOptions);
    let argsAfterDoubleDashes = [];
    const doubleDashesIndex = argv.indexOf("--");
    if (doubleDashesIndex > -1) {
      argsAfterDoubleDashes = argv.slice(doubleDashesIndex + 1);
      argv = argv.slice(0, doubleDashesIndex);
    }
    let parsed = mri2(argv, mriOptions);
    parsed = Object.keys(parsed).reduce((res, name) => {
      return __assign(__assign({}, res), {
        [camelcaseOptionName(name)]: parsed[name]
      });
    }, {_: []});
    const args = parsed._;
    const options = {
      "--": argsAfterDoubleDashes
    };
    const ignoreDefault = command && command.config.ignoreOptionDefaultValue ? command.config.ignoreOptionDefaultValue : this.globalCommand.config.ignoreOptionDefaultValue;
    let transforms = Object.create(null);
    for (const cliOption of cliOptions) {
      if (!ignoreDefault && cliOption.config.default !== void 0) {
        for (const name of cliOption.names) {
          options[name] = cliOption.config.default;
        }
      }
      if (Array.isArray(cliOption.config.type)) {
        if (transforms[cliOption.name] === void 0) {
          transforms[cliOption.name] = Object.create(null);
          transforms[cliOption.name]["shouldTransform"] = true;
          transforms[cliOption.name]["transformFunction"] = cliOption.config.type[0];
        }
      }
    }
    for (const key of Object.keys(parsed)) {
      if (key !== "_") {
        const keys = key.split(".");
        setDotProp(options, keys, parsed[key]);
        setByType(options, transforms);
      }
    }
    return {
      args,
      options
    };
  }
  runMatchedCommand() {
    const {args, options, matchedCommand: command} = this;
    if (!command || !command.commandAction)
      return;
    command.checkUnknownOptions();
    command.checkOptionValue();
    command.checkRequiredArgs();
    const actionArgs = [];
    command.args.forEach((arg, index) => {
      if (arg.variadic) {
        actionArgs.push(args.slice(index));
      } else {
        actionArgs.push(args[index]);
      }
    });
    actionArgs.push(options);
    return command.commandAction.apply(this, actionArgs);
  }
}

const cac = (name = "") => new CAC(name);

const name = "renames";
const version = "1.0.2";
const packageManager = "pnpm@7.5.0";
const description = "cli-renames";
const keywords = [
	"cli",
	"typescript",
	"renames"
];
const license = "MIT";
const author = "peterroe";
const module$1 = "./dist/index.mjs";
const main$1 = "./dist/index.mjs";
const bin = {
	renames: "./dist/index.mjs"
};
const exports$1 = {
	".": {
		require: "./dist/index.cjs",
		"import": "./dist/index.mjs"
	}
};
const scripts = {
	build: "unbuild",
	lint: "eslint \"**/*.{ts,js}\"",
	fix: "eslint --fix \"**/*.{ts,js}\"",
	start: "esno src/index.ts",
	test: "vitest",
	release: "np"
};
const devDependencies = {
	"@antfu/eslint-config": "0.25.2",
	cac: "6.7.12",
	eslint: "8.19.0",
	esno: "0.16.3",
	husky: "4.3.8",
	np: "7.6.2",
	typescript: "4.7.4",
	unbuild: "0.7.4",
	vite: "2.9.13",
	vitest: "0.18.0"
};
const husky = {
	hooks: {
		"pre-commit": "npm run lint"
	}
};
const dependencies = {
	chalk: "^5.0.1",
	debug: "^4.3.4"
};
const pkg = {
	name: name,
	version: version,
	packageManager: packageManager,
	description: description,
	keywords: keywords,
	license: license,
	author: author,
	module: module$1,
	main: main$1,
	bin: bin,
	exports: exports$1,
	scripts: scripts,
	devDependencies: devDependencies,
	husky: husky,
	dependencies: dependencies
};

function logDiff(nameMap) {
  const maxNameLen = nameMap.reduce((pre, cur) => {
    return Math.max(pre, cur[0].length);
  }, 0);
  nameMap.forEach(([before, after]) => {
    before = before.padEnd(maxNameLen, " ");
    console.log(`${chalk__default.red(before)} => ${chalk__default.green(after)}`);
  });
}

const tLog = debug__default("compiler:tokenize");
const gLog = debug__default("compiler:generatetokenArrayMap");
const pLog = debug__default("compiler:parse");
const fLog = debug__default("compiler:fixGap");
const sLog = debug__default("compiler:synthesize");
class Compiler {
  constructor(beforePatten, afterPatten, targetStr) {
    this.beforePatten = beforePatten;
    this.afterPatten = afterPatten;
    this.targetStr = targetStr;
    tLog("--- bofore tokenize start ---");
    this.beforeTokens = this.tokenize({ i: 0, str: beforePatten });
    tLog({ beforeTokens: this.beforeTokens });
    this.afterTokens = this.tokenize({ i: 0, str: afterPatten });
    tLog({ afterTokens: this.afterTokens });
    tLog("--- after tokenize end ---");
    this.fixGap();
    this.syntaxArray = this.synthesize();
  }
  tokenize({ i, str }) {
    const tokens = [];
    const len = str.length;
    while (i < len) {
      if (str[i] === "(") {
        const nextI = str.indexOf(")", i);
        const t = this.tokenize({
          i: 0,
          str: str.substring(i + 1, nextI)
        });
        tokens.push(t);
        i = nextI + 1;
      } else if (/^\[\w-\w\]/.test(str.slice(i))) {
        const t = /^\[(\w-\w)\]/.exec(str)[1];
        tokens.push(t);
        i += 5;
      } else if (str[i] === "?") {
        tokens.push("?");
        i++;
      } else if (str[i] === "*") {
        tokens.push("*");
        i++;
      } else if (str[i] === "+") {
        tokens.push("+");
        i++;
      } else if (str.slice(i).includes("(")) {
        const index = str.indexOf("(", i);
        tokens.push(str.substring(i, index));
        i = index;
      } else {
        tokens.push(str.slice(i));
        break;
      }
    }
    return tokens;
  }
  generatetokenArrayMap() {
    const { beforeTokens, afterTokens } = this;
    const tokenArrayMap = [];
    for (let i = 0; i < beforeTokens.length; i++) {
      const beforeToken = beforeTokens[i];
      const afterToken = afterTokens[i];
      gLog({ i, beforeToken, afterToken });
      if (Array.isArray(beforeToken)) {
        const map = [];
        for (let j = 0; j < beforeToken.length; j++) {
          const b = beforeToken[j];
          const a = afterToken[j];
          map.push([b, a]);
        }
        tokenArrayMap.push(map);
      } else {
        tokenArrayMap.push(/* @__PURE__ */ new Map([[beforeToken, afterToken]]));
      }
    }
    return tokenArrayMap;
  }
  fixGap() {
    fLog("--- fixGap start ---");
    const { beforeTokens, afterTokens } = this;
    fLog({ beforeTokens, afterTokens });
    let i = 0;
    while (i < beforeTokens.length) {
      const b = beforeTokens[i];
      const a = afterTokens[i];
      if (Array.isArray(a) && typeof b === "string")
        afterTokens.splice(i, 0, "");
      else if (Array.isArray(b) && typeof a === "string")
        beforeTokens.splice(i, 0, "");
      i++;
    }
    this.beforeTokens = beforeTokens;
    this.afterTokens = afterTokens;
    fLog({ beforeTokens, afterTokens });
    fLog("--- fixGap end ---");
  }
  synthesize() {
    sLog("--- synthesize start ---");
    const { beforeTokens, afterTokens, targetStr } = this;
    sLog({ beforeTokens, afterTokens });
    const regStrArr = [];
    for (let i = 0; i < beforeTokens.length; i++) {
      const beforeToken = beforeTokens[i];
      if (Array.isArray(beforeToken)) {
        const regStr = `([${beforeToken[0]}]${beforeToken[1] ? beforeToken[1] : ""})`;
        regStrArr.push(regStr);
      } else {
        regStrArr.push(`(${beforeToken})`);
      }
    }
    const reg = new RegExp(regStrArr.join(""));
    const syntaxArray = reg.exec(targetStr)?.slice(1);
    sLog({ reg, syntaxArray, targetStr });
    sLog("--- synthesize end ---");
    return syntaxArray;
  }
  parse() {
    const syntaxArray = this.syntaxArray;
    if (!syntaxArray)
      return;
    const beforeTokens = this.beforeTokens;
    const afterTokens = this.afterTokens;
    pLog("--- parse start ---");
    pLog({ syntaxArray, beforeTokens, afterTokens });
    for (let i = 0; i < syntaxArray.length; i++) {
      const itemToken = afterTokens[i];
      if (Array.isArray(itemToken)) {
        const mode = itemToken[0];
        if (/[A-Z]-[A-Z]/.test(mode))
          syntaxArray[i] = syntaxArray[i].toUpperCase();
        else if (/[a-z]-[a-z]/.test(mode))
          syntaxArray[i] = syntaxArray[i].toLowerCase();
        else if (!mode)
          syntaxArray[i] = "";
      } else {
        syntaxArray[i] = itemToken;
      }
    }
    pLog({ syntaxArray });
    pLog("--- parse end ---");
    return syntaxArray.join("");
  }
}
console.log(new Compiler("([a-z]?)([A-Z])([a-z]*).js", "()([A-Z])([a-z]*).js", "cHelloword.js").parse());

function rename(beforePatten, afterPatten, files) {
  const result = [];
  files.forEach((file) => {
    const afterName = new Compiler(beforePatten, afterPatten, file).parse();
    if (afterName)
      result.push([file, afterName]);
  });
  return result;
}

function renameFile(nameMap, dir) {
  nameMap.forEach(([oldFileName, newFileName]) => {
    fs__default.renameSync(path__default.join(dir, oldFileName), path__default.join(dir, newFileName));
  });
}

const cli = cac();
cli.option("--dir <dir>", "target dir", {
  default: "./"
});
cli.option("--write", "write to file");
cli.help();
cli.usage("<beforePatten> <afterPatten> --dir <dir>");
cli.version(pkg.version);
function getParams() {
  const { args, options } = cli.parse();
  const [beforePatten, afterPatten] = args;
  const { dir, write } = options;
  if (!dir || !beforePatten || !afterPatten)
    process.exit(1);
  const files = fs__default.readdirSync(dir);
  return {
    beforePatten,
    afterPatten,
    files,
    dir,
    write
  };
}
function main() {
  const {
    beforePatten,
    afterPatten,
    files,
    dir,
    write
  } = getParams();
  const nameMap = rename(beforePatten, afterPatten, files);
  if (!write)
    logDiff(nameMap);
  else
    renameFile(nameMap, dir);
}
main();
